package com.nantian.cloud.jljc.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.nantian.cloud.jljc.entity.DictEnterpriseType;
import com.nantian.cloud.jljc.service.DictEnterpriseTypeService;
import com.nantian.sc.cloud.common.core.util.R;
import com.nantian.sc.cloud.common.log.annotation.SysLog;
import com.nantian.sc.cloud.common.security.service.ScUser;
import com.nantian.sc.cloud.common.security.util.SecurityUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.Date;


/**
 * 企业类型
 *
 * @author
 * @date 2020-08-14
 */
@CrossOrigin
@RestController
@AllArgsConstructor
@RequestMapping("/dictEnterpriseType")
@Api(value="dictEnterpriseType",tags = "企业类型")
public class DictEnterpriseTypeController {
	private final DictEnterpriseTypeService dictEnterpriseTypeService;


	/**
	 * 分页查询
	 *
	 * @param page      分页对象
	 * @param dictEnterpriseType
	 * @return
	 */
	@GetMapping("/page")
	@ApiOperation(value = "分页获取企业类型", notes = "分页获取企业类型")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "page", value = "分页对象", required = false, dataType = "json", paramType = "path"),
			@ApiImplicitParam(name = "dictEnterpriseType", value = "企业类型对象", required = false, dataType = "json", paramType = "path")
	})
	public R getAdvancedMaterialPage(Page page, DictEnterpriseType dictEnterpriseType) {
		dictEnterpriseType.setDelFlag("1");  //筛选未删除项
		return R.ok(dictEnterpriseTypeService.page(page, Wrappers.query(dictEnterpriseType)));
	}


	/**
	 * 信息
	 *
	 * @param dictEnterpriseTypeId
	 * @return R
	 */
	@GetMapping("/{dictEnterpriseTypeId}")
	@ApiOperation(value = "根据id获取企业类型", notes = "根据id获取企业类型")
	@ApiImplicitParam(name = "id", value = "id", required = true, dataType = "int", paramType = "path")
	public R getById(@PathVariable("dictEnterpriseTypeId") Integer  dictEnterpriseTypeId) {
		return R.ok(dictEnterpriseTypeService.getById(dictEnterpriseTypeId));
	}

	/**
	 * 保存
	 *
	 * @param dictEnterpriseType
	 * @return R
	 */
	@PostMapping
	@SysLog("新增企业类型")
	@ApiOperation(value = "新增企业类型", notes = "新增企业类型")
	@ApiImplicitParam(name = "dictEnterpriseType", value = "企业类型对象", required = true, dataType = "json", paramType = "path")
	public R save(@RequestBody DictEnterpriseType dictEnterpriseType) {
		ScUser u= SecurityUtils.getUser();
		dictEnterpriseType.setUserid(u.getId());
		dictEnterpriseType.setDeptId(u.getDeptId());
		dictEnterpriseType.setUpdateTime(new Date());
		dictEnterpriseType.setDelFlag("1");
		return R.ok(dictEnterpriseTypeService.save(dictEnterpriseType));
	}

	/**
	 * 修改
	 *
	 * @param dictEnterpriseType
	 * @return R
	 */
	@PutMapping
	@SysLog("修改企业类型")
	@ApiOperation(value = "修改企业类型", notes = "修改企业类型")
	@ApiImplicitParam(name = "dictEnterpriseType", value = "企业类型对象", required = true, dataType = "json", paramType = "path")
	public R updateById(@RequestBody DictEnterpriseType dictEnterpriseType) {
		ScUser u= SecurityUtils.getUser();
		dictEnterpriseType.setUserid(u.getId());
		dictEnterpriseType.setUpdateTime(new Date());
		return R.ok(dictEnterpriseTypeService.updateById(dictEnterpriseType));
	}

	/**
	 * 删除
	 *
	 * @param dictEnterpriseTypeId
	 * @return R
	 */
	@DeleteMapping("/{dictEnterpriseTypeId}")
	@SysLog("删除企业类型")
	@ApiOperation(value = "删除企业类型", notes = "删除企业类型")
	@ApiImplicitParam(name = "id", value = "id", required = true, dataType = "int", paramType = "path")
	public R removeById(@PathVariable Integer dictEnterpriseTypeId) {
		DictEnterpriseType dictEnterpriseType = dictEnterpriseTypeService.getById(dictEnterpriseTypeId);
		if(dictEnterpriseType==null){
			return R.failed("企业类型不存在，删除失败！");
		}
		ScUser u= SecurityUtils.getUser();
		dictEnterpriseType.setUserid(u.getId());
		dictEnterpriseType.setUpdateTime(new Date());
		dictEnterpriseType.setDelFlag("4");
		return R.ok(dictEnterpriseTypeService.updateById(dictEnterpriseType));
	}

}
