package com.nantian.cloud.jljc.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nantian.cloud.jljc.entity.TFaAccreditingBody;
import com.nantian.cloud.jljc.entity.TFaFieldReconnaissance;
import com.nantian.cloud.jljc.entity.TFaFixedAssets;
import com.nantian.cloud.jljc.entity.TFaSpecialistOpinion;
import com.nantian.cloud.jljc.service.TFaAccreditingBodyService;
import com.nantian.cloud.jljc.service.TFaFixedAssetsService;
import com.nantian.cloud.jljc.service.TFaSpecialistOpinionService;
import com.nantian.sc.cloud.common.core.util.R;
import com.nantian.sc.cloud.common.log.annotation.SysLog;
import com.nantian.sc.cloud.common.minio.service.MinioTemplate;
import com.nantian.sc.cloud.common.security.service.ScUser;
import com.nantian.sc.cloud.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import org.apache.commons.io.FilenameUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.UUID;

/**
 * <p>
 * 固定资产投资项目节能审查专家及专家组意见表 前端控制器
 * </p>
 *
 * @author fyf
 * @since 2022-02-15
 */
@CrossOrigin
@AllArgsConstructor
@RestController
@RequestMapping("/tFaSpecialistOpinion")
public class TFaSpecialistOpinionController {

    @Autowired
    MinioTemplate minioTemplate;

    @Autowired
    TFaSpecialistOpinionService tFaSpecialistOpinionService;

    @Autowired
    TFaAccreditingBodyService tFaAccreditingBodyService;

    @Autowired
    TFaFixedAssetsService tFaFixedAssetsService;


    @PostMapping("/fileUpload/{id}")
    @SysLog("根据id上传固定资产投资项目节能审查专家及专家组意见表")
    public R fileUpload(@PathVariable Integer id,@RequestParam("file") MultipartFile[] multipartFiles) throws Exception {
        TFaAccreditingBody tFaAccreditingBody = tFaAccreditingBodyService.getOne(new QueryWrapper<TFaAccreditingBody>().eq("project_id", id));
        ScUser u = SecurityUtils.getUser();
        if (multipartFiles.length > 0) {
            for (MultipartFile multipartFile : multipartFiles) {
                //获取文件名称
                String fileName = multipartFile.getOriginalFilename();
                //获取文件的后缀
                String extension = "." + FilenameUtils.getExtension(multipartFile.getOriginalFilename());
                //生成新的文件名称
                String newFileName = "ZJ" + new SimpleDateFormat("yyyyMMddHHmmss").format(new Date())
                        + UUID.randomUUID().toString()
                        .replace("-", "") + extension;
                //文件的大小
                long size = multipartFile.getSize();
                //文件的类型
                String contentType = multipartFile.getContentType();
                //上传文件
                InputStream inputStream = multipartFile.getInputStream();
                minioTemplate.putObject("jljc", newFileName, inputStream, inputStream.available(), "application/octet-stream");
                //文件信息保存到数据库
                TFaSpecialistOpinion specialistOpinion = new TFaSpecialistOpinion();
                specialistOpinion.setDelFlag(1);
                specialistOpinion.setExtension(extension);
                specialistOpinion.setFilePath("jljc/" + newFileName);
                specialistOpinion.setFileSize(String.valueOf(size));
                specialistOpinion.setFileType(contentType);
                specialistOpinion.setNewFileName(newFileName);
                specialistOpinion.setOldFileName(fileName);
                specialistOpinion.setDeptId(u.getDeptId());
                specialistOpinion.setUpdateTime(new Date());
                specialistOpinion.setUserId(u.getId());
                specialistOpinion.setUploadPerson(u.getUserRealName());
                specialistOpinion.setProjectId(id);
                specialistOpinion.setUploadTime(new Date());
                specialistOpinion.setPersonId(tFaAccreditingBody.getId());
                tFaSpecialistOpinionService.save(specialistOpinion);
                return R.ok();
            }
        } else {
            return R.failed("上传文件为空！");
        }
        return R.ok();
    }

    @GetMapping("/getFileList/{id}")
    @SysLog("根据id查询固定资产投资项目节能审查专家及专家组意见表")
    public R getFileList(@PathVariable Integer id){
        List<TFaSpecialistOpinion> list = tFaSpecialistOpinionService.list(new QueryWrapper<TFaSpecialistOpinion>()
                .eq("del_flag", 1)
                .eq("project_id", id));
        return R.ok(list);
    }

    @GetMapping("/fileDownload/{id}")
    @SysLog("根据id下载固定资产投资项目节能审查专家及专家组意见表")
    public R fileDownload(@PathVariable("id") Integer id) throws IOException {
        TFaSpecialistOpinion tFaSpecialistOpinion = tFaSpecialistOpinionService.getById(id);
        //获取文件名
        String newFileName = tFaSpecialistOpinion.getNewFileName();
        //文件下载
        minioTemplate.getObject("jljc",newFileName);
        return R.ok();
    }

    @DeleteMapping("/fileRemove/{id}")
    @SysLog("根据id删除固定资产投资项目节能审查专家及专家组意见表")
    public R fileRemove(@PathVariable("id") String id) throws Exception {
        ScUser u = SecurityUtils.getUser();
        TFaSpecialistOpinion tFaSpecialistOpinion = tFaSpecialistOpinionService.getById(id);
        //获取文件名
        String newFileName = tFaSpecialistOpinion.getNewFileName();
        tFaSpecialistOpinion.setDelFlag(4);
        tFaSpecialistOpinion.setUserId(u.getId());
        tFaSpecialistOpinion.setUpdateTime(new Date());
        tFaSpecialistOpinion.setDeptId(u.getDeptId());
        tFaSpecialistOpinionService.updateById(tFaSpecialistOpinion);
        //文件删除
        minioTemplate.removeObject("jljc",newFileName);
        return R.ok();
    }
}
