package com.nantian.cloud.jljc.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nantian.cloud.jljc.entity.TFaAccreditingBody;
import com.nantian.cloud.jljc.entity.TFaFieldReconnaissance;
import com.nantian.cloud.jljc.entity.TFaSpecialistOpinion;
import com.nantian.cloud.jljc.entity.TFaTechnologyReport;
import com.nantian.cloud.jljc.service.TFaAccreditingBodyService;
import com.nantian.cloud.jljc.service.TFaTechnologyReportService;
import com.nantian.sc.cloud.common.core.util.R;
import com.nantian.sc.cloud.common.log.annotation.SysLog;
import com.nantian.sc.cloud.common.minio.service.MinioTemplate;
import com.nantian.sc.cloud.common.security.service.ScUser;
import com.nantian.sc.cloud.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import org.apache.commons.io.FilenameUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.UUID;

/**
 * <p>
 * 固定资产投资项目节能审查技术审查报告表 前端控制器
 * </p>
 *
 * @author fyf
 * @since 2022-02-15
 */
@CrossOrigin
@AllArgsConstructor
@RestController
@RequestMapping("/tFaTechnologyReport")
public class TFaTechnologyReportController {

    @Autowired
    MinioTemplate minioTemplate;

    @Autowired
    TFaTechnologyReportService tFaTechnologyReportService;

    @Autowired
    TFaAccreditingBodyService tFaAccreditingBodyService;

    @PostMapping("/fileUpload/{id}")
    @SysLog("根据id上传固定资产投资项目节能审查技术审查报告表")
    public R fileUpload(@PathVariable Integer id,@RequestParam("file") MultipartFile[] multipartFiles) throws Exception {
        TFaAccreditingBody tFaAccreditingBody = tFaAccreditingBodyService.getOne(new QueryWrapper<TFaAccreditingBody>().eq("project_id", id));
        ScUser u = SecurityUtils.getUser();
        if (multipartFiles.length > 0) {
            for (MultipartFile multipartFile : multipartFiles) {
                //获取文件名称
                String fileName = multipartFile.getOriginalFilename();
                //获取文件的后缀
                String extension = "." + FilenameUtils.getExtension(multipartFile.getOriginalFilename());
                //生成新的文件名称
                String newFileName = "JSSC" + new SimpleDateFormat("yyyyMMddHHmmss").format(new Date())
                        + UUID.randomUUID().toString()
                        .replace("-", "") + extension;
                //文件的大小
                long size = multipartFile.getSize();
                //文件的类型
                String contentType = multipartFile.getContentType();
                //上传文件
                InputStream inputStream = multipartFile.getInputStream();
                minioTemplate.putObject("jljc", newFileName, inputStream, inputStream.available(), "application/octet-stream");
                //文件信息保存到数据库
                TFaTechnologyReport technologyReport = new TFaTechnologyReport();
                technologyReport.setDelFlag(1);
                technologyReport.setExtension(extension);
                technologyReport.setFilePath("jljc/" + newFileName);
                technologyReport.setFileSize(String.valueOf(size));
                technologyReport.setFileType(contentType);
                technologyReport.setNewFileName(newFileName);
                technologyReport.setOldFileName(fileName);
                technologyReport.setDeptId(u.getDeptId());
                technologyReport.setUpdateTime(new Date());
                technologyReport.setUserId(u.getId());
                technologyReport.setUploadPerson(u.getUserRealName());
                technologyReport.setProjectId(id);
                technologyReport.setUploadTime(new Date());
                technologyReport.setPersonId(tFaAccreditingBody.getId());
                tFaTechnologyReportService.save(technologyReport);
                return R.ok();
            }

        } else {
            return R.failed("上传文件为空！");
        }
        return R.ok();
    }

    @GetMapping("/fileDownload/{id}")
    @SysLog("根据id下载固定资产投资项目节能审查技术审查报告表")
    public R fileDownload(@PathVariable("id") String id) throws IOException {
        TFaTechnologyReport tFaTechnologyReport = tFaTechnologyReportService.getById(id);
        //获取文件名
        String newFileName = tFaTechnologyReport.getNewFileName();
        //文件下载
        minioTemplate.getObject("jljc",newFileName);
        return R.ok();
    }

    @GetMapping("/getFileList/{id}")
    @SysLog("根据id查询固定资产投资项目节能审查技术审查报告表")
    public R getFileList(@PathVariable Integer id){

        List<TFaTechnologyReport> list = tFaTechnologyReportService.list(
                new QueryWrapper<TFaTechnologyReport>()
                        .eq("del_flag", 1)
                        .eq("project_id", id));
        return R.ok(list);
    }

    @DeleteMapping("/fileRemove/{id}")
    @SysLog("根据id删除固定资产投资项目节能审查技术审查报告表")
    public R fileRemove(@PathVariable("id") String id) throws Exception {
        TFaTechnologyReport tFaTechnologyReport = tFaTechnologyReportService.getById(id);
        ScUser u = SecurityUtils.getUser();
        //获取文件名
        String newFileName = tFaTechnologyReport.getNewFileName();
        tFaTechnologyReport.setDelFlag(4);
        tFaTechnologyReport.setUserId(u.getId());
        tFaTechnologyReport.setUpdateTime(new Date());
        tFaTechnologyReport.setDeptId(u.getDeptId());
        tFaTechnologyReportService.updateById(tFaTechnologyReport);
        //文件删除
        minioTemplate.removeObject("jljc",newFileName);
        return R.ok();
    }

}
