package com.nantian.cloud.jljc.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.time.LocalDate;
import java.io.Serializable;
import java.util.Date;

/**
 * <p>
 * 节能减排项目
 * </p>
 *
 * @author stz
 * @since 2020-08-09
 */
@Data
@TableName("reduce_emission")
@EqualsAndHashCode(callSuper = true)
@ApiModel(value = "节能减排项目")
public class ReduceEmission extends Model<ReduceEmission> {

    private static final long serialVersionUID = 1L;

    /**
     * 主键id
     */
    @TableId(value = "id")
    @ApiModelProperty(value = "主键id")
    private Long id;

    /**
     * 企业社会信用代码
     */
    @ApiModelProperty(value = "企业社会信用代码")
    private String enterpriseCode;

    /**
     * 行业
     */
    @ApiModelProperty(value = "行业")
    private String tradeName;

    /**
     * 地区
     */
    @ApiModelProperty(value = "地区")
    private String regionName;

    /**
     * 项目名称
     */
    @ApiModelProperty(value = "项目名称")
    private String projectName;

    /**
     * 项目介绍
     */
    @ApiModelProperty(value = "项目介绍")
    private String projectExplain;

    /**
     * 项目附件文件路径
     */
    @ApiModelProperty(value = "项目附件文件路径")
    private String projectFileUrl;

    /**
     * 项目来源（1：能源审计，2：节能考核）
     */
    @ApiModelProperty(value = "项目来源，" +
            "1：能源审计，" +
            "2：节能考核")
    private String projectSource;

    /**
     * 项目负责人
     */
    @ApiModelProperty(value = "项目负责人")
    private String leader;

    /**
     * 项目负责人电话
     */
    @ApiModelProperty(value = "项目负责人电话")
    private String mobile;

    /**
     * 项目开始时间
     */
    @ApiModelProperty(value = "项目开始时间")
    private LocalDate startDate;

    /**
     * 项目结束时间
     */
    @ApiModelProperty(value = "项目结束时间")
    private LocalDate endDate;

    /**
     * 耗能类型（110电力，120水，...）
     */
    @ApiModelProperty(value = "耗能类型（110电力，120水，...）")
    private String energyConsumeType;

    /**
     * 目标总量
     */
    @ApiModelProperty(value = "目标总量")
    private String total;

    /**
     * 目标能耗
     */
    @ApiModelProperty(value = "目标能耗")
    private String energyConsume;

    /**
     * 目标强度
     */
    @ApiModelProperty(value = "目标强度")
    private String strength;

    /**
     * 目标碳排放
     */
    @ApiModelProperty(value = "目标碳排放")
    private String carbonEmission;

    /**
     * 实际总量
     */
    @ApiModelProperty(value = "实际总量")
    private String realTotal;

    /**
     * 实际强度
     */
    @ApiModelProperty(value = "实际强度")
    private String realStrength;

    /**
     * 实际能耗
     */
    @ApiModelProperty(value = "实际能耗")
    private String realEnergyConsume;

    /**
     * 实际碳排放
     */
    @ApiModelProperty(value = "实际碳排放")
    private String realCarbonEmission;

    /**
     * 推荐状态（1：推荐，2：不推荐）
     */
    @ApiModelProperty(value = "推荐状态，" +
            "1：推荐，" +
            "2：不推荐")
    private String recommendStatus;

    /**
     * 审核人id
     */
    @ApiModelProperty(value = "审核人id")
    private Long auditorId;

    /**
     * 审核意见
     */
    @ApiModelProperty(value = "审核意见")
    private String auditRemark;

    /**
     * 审核时间
     */
    @ApiModelProperty(value = "审核时间")
    private LocalDate auditDate;

    /**
     * 评分
     */
    @ApiModelProperty(value = "评分")
    private String auditGrade;

    /**
     * 用户id
     */
    @ApiModelProperty(value = "用户id")
    private Integer userid;

    /**
     * 更新时间
     */
    @ApiModelProperty(value = "更新时间")
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern="yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date updateTime;

    /**
     * 注销状态
     */
    @ApiModelProperty(value = "注销状态（1：未删除，4：已删除）")
    private String delFlag;

    /**
     * 部门id
     */
    @ApiModelProperty(value = "部门id")
    private Integer deptId;

    /**
     * 主键值
     */
    @Override
    protected Serializable pkVal() {
        return this.id;
    }
}
