package com.nantian.cloud.jly.service.impl;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.nantian.cloud.jly.entity.CheckLevelDegreeReport;
import com.nantian.cloud.jly.entity.InspectionData;
import com.nantian.cloud.jly.mapper.CheckLevelDegreeReportMapper;
import com.nantian.cloud.jly.service.CheckLevelDegreeReportService;
import com.nantian.cloud.jly.service.InspectionDataService;
import com.nantian.sc.cloud.common.core.util.R;
import com.nantian.sc.cloud.common.data.datascope.DataScope;
import com.nantian.sc.cloud.common.security.service.ScUser;
import com.nantian.sc.cloud.common.security.util.SecurityUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.math.BigInteger;
import java.util.Date;


@Service("checkLevelDegreeReportService")
public class CheckLevelDegreeReportServiceImpl extends ServiceImpl<CheckLevelDegreeReportMapper, CheckLevelDegreeReport> implements CheckLevelDegreeReportService {

    @Autowired
    private InspectionDataService inspectionDataService;

    private static final String EXTENSION_XLS = "xls";
    private static final String EXTENSION_XLSX = "xlsx";

    @Override
    public IPage getPageScope(Page page, CheckLevelDegreeReport checkLevelDegreeReport) {
        return baseMapper.getListPage(page,checkLevelDegreeReport);
    }

    @Override
    public R excelImport(MultipartFile file, BigInteger checkPlanId, String epc, InspectionData inspectionData) {
        if (file == null) {
            //throw new FileFormatException("");
            return R.failed("导入失败，传入文件不存在！");
        }

        //获取原文件
        String originalFileName = file.getOriginalFilename();

        //判断传入文件格式
        if (!(originalFileName.endsWith(EXTENSION_XLS) || originalFileName.endsWith(EXTENSION_XLSX))) {
            //throw new FileFormatException("传入文件不是excel格式!");
            return R.failed("传入文件不是excel格式!");
        }
        Workbook workbook = null;
        try {
            if (originalFileName.endsWith(EXTENSION_XLS)) {
                workbook = new HSSFWorkbook(file.getInputStream());
            } else if (originalFileName.endsWith(EXTENSION_XLSX)) {
                workbook = new XSSFWorkbook(file.getInputStream());
            }

            //读文件
            for (int numSheet = 0; numSheet < workbook.getNumberOfSheets(); numSheet++) {
                org.apache.poi.ss.usermodel.Sheet sheet = workbook.getSheetAt(numSheet);
                if (sheet == null) {
                    continue;
                }
                int firstRowIndex = sheet.getFirstRowNum();
                int lastRowIndex = sheet.getLastRowNum();
                //读取首行（表头）
                Row firstRow = sheet.getRow(firstRowIndex);
                for (int i = firstRow.getFirstCellNum(); i <= firstRow.getLastCellNum(); i++) {
                    Cell cell = firstRow.getCell(i);
                    String cellValue = this.getCellValue(cell, true);
                    System.err.println("表头数据：" + cellValue + "\t");
                }

                System.err.println("开始导入数据...");
                CheckLevelDegreeReport checkLevelDegreeReport = new CheckLevelDegreeReport();
                //从第3行开始才有数据
                for (int rowIndex = firstRowIndex + 2; rowIndex <= lastRowIndex; rowIndex++) {
                    Row currentRow = sheet.getRow(rowIndex);
                    int firstColumnIndex = currentRow.getFirstCellNum(); //首列
                    int lastColumnIndex = currentRow.getLastCellNum(); //尾列
                    boolean s = false;

                    for (int columnIndex = firstColumnIndex; columnIndex <= lastColumnIndex; columnIndex++) {
                        Cell currentCell = currentRow.getCell(columnIndex);
                        String currentCellValue = this.getCellValue(currentCell, true);
                        System.err.println(" " + currentCellValue + "\t");
                        if (columnIndex == firstColumnIndex) {
                            checkLevelDegreeReport.setType(currentCellValue);
                        }
                        if (columnIndex == firstColumnIndex + 1) {
                            checkLevelDegreeReport.setPurpose(currentCellValue);
                        }
                        if (columnIndex == firstColumnIndex + 2) {
                            checkLevelDegreeReport.setInoutLevel(currentCellValue);
                        }
                        if (columnIndex == firstColumnIndex + 3) {
                            checkLevelDegreeReport.setInoutDegree(currentCellValue);
                        }
                        if (columnIndex == firstColumnIndex + 4) {
                            checkLevelDegreeReport.setSecondaryLevel(currentCellValue);
                        }
                        if (columnIndex == firstColumnIndex + 5) {
                            checkLevelDegreeReport.setSecondaryDegree(currentCellValue);
                        }
                        if (columnIndex == firstColumnIndex + 6) {
                            checkLevelDegreeReport.setMainLevel(currentCellValue);
                        }
                        if (columnIndex == firstColumnIndex + 7) {
                            checkLevelDegreeReport.setMainDegree(currentCellValue);
                        }
                        if (columnIndex == firstColumnIndex + 8) {
                            checkLevelDegreeReport.setOtherLevel(currentCellValue);
                        }
                        if (columnIndex == firstColumnIndex + 9) {
                            checkLevelDegreeReport.setOtherDegree(currentCellValue);
                        }
                    }
                    checkLevelDegreeReport.setCheckPlanId(checkPlanId.longValue());
                    checkLevelDegreeReport.setEpc(epc);
                    ScUser u = SecurityUtils.getUser();
                    checkLevelDegreeReport.setUserid(u.getId());
                    checkLevelDegreeReport.setDeptId(u.getDeptId());
                    checkLevelDegreeReport.setDelFlag("1");
                    checkLevelDegreeReport.setUpdateTime(new Date());
                    this.save(checkLevelDegreeReport);

                    inspectionData.setAccuracyGrade("Y");
                    inspectionDataService.updateById(inspectionData);  //更新中间表
                }
            }
            return R.ok("导入成功");
        } catch (Exception e) {
            System.err.println(e.getMessage());
            e.printStackTrace();
            return R.failed("导入失败！" + e.getMessage());
        } finally {
            if (workbook != null) {
                try {
                    file.getInputStream().close();
                    workbook.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    private String getCellValue(Cell cell, boolean b) {
        if (cell == null) {
            return "";
        }
        if (b) {
            //虽然excel中设置的都是文本，但是数字文本还被读错，如“1”取成“1.0”,加上下面这句，临时把它当做文本来读取
            cell.setCellType(Cell.CELL_TYPE_STRING);
        }
        if (cell.getCellType() == Cell.CELL_TYPE_BOOLEAN) {
            return String.valueOf(cell.getBooleanCellValue());
        } else if (cell.getCellType() == Cell.CELL_TYPE_NUMERIC) {
            return String.valueOf(cell.getNumericCellValue());
        } else {
            return String.valueOf(cell.getStringCellValue());
        }
    }


}
